<?php

error_reporting(0xFFFFFFFF);
define('COOKIE_FILE', './cookie.tmp');

// IMToken
$auth = '01234567890abcdef01234567890abcdef012345';

// sender
$uin = 123456;

// recipient
$to_uin = 654321;

// transferred file
$file_name = 'example.zip';

/*******************************************************************************
 * Helper functions.
 ******************************************************************************/

$file_contents = file_get_contents($file_name);
$file_size = strlen($file_contents);

class HeadReader
{
	private $headers;
	
	public function __construct()
	{
		$this->headers = array();
	}
	
	public function read($c, $header)
	{
		$len = strlen($header);
		$header = explode(': ', $header, 2);
		if (count($header) != 2)
			return $len;
		$this->headers[$header[0]] = trim($header[1]);
		return $len;
	}
	
	public function get($name)
	{
		if (!isset($this->headers[$name]))
			return null;
		return $this->headers[$name];
	}
}

function check_status($ret)
{
	if (is_null($ret))
		return false;
	if (!isset($ret->result) || !isset($ret->result->status))
		return false;
	if (!is_int($ret->result->status))
		return false;
	return ($ret->result->status == 0);
}

/*******************************************************************************
 * Authentication (once per session).
 ******************************************************************************/

echo("Authentication...\n"); flush();

$url = 'https://drive.mpa.gg.pl/signin';

$data = json_encode(array(
	"id" => "0123456789abcdef0123456789abcdef",
	"name" => "Hostname",
	"os_version" => "WINNT x86-msvc",
	"client_version" => "11.0.0.8169",
	"type" => "desktop"
));

$headers = array(
	'Authorization: IMToken ' . $auth,
	'X-gged-user: gg/pl:' . $uin,
	'X-gged-client-metadata: ' . $data,
	'X-gged-api-version: 6',
);

$c = curl_init();
$hreader = new HeadReader();

curl_setopt($c, CURLOPT_URL, $url);
curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
curl_setopt($c, CURLOPT_HTTPHEADER, $headers);
curl_setopt($c, CURLOPT_PUT, true);
curl_setopt($c, CURLOPT_COOKIEJAR, COOKIE_FILE);
curl_setopt($c, CURLOPT_HEADERFUNCTION, array(&$hreader, 'read'));

$ret = curl_exec($c);

curl_close($c);

if (!$ret)
	die("Authentication failed\n");
$ret = json_decode($ret);
if (!check_status($ret))
	die("Authentication failed\n");

$security_token = $hreader->get('X-gged-security-token');
if (!$security_token)
	die("Didn't received security_token\n");

/*******************************************************************************
 * Offering a file transfer.
 ******************************************************************************/

echo("Offering a file transfer...\n"); flush();

$url = 'https://drive.mpa.gg.pl/send_ticket';

$data = json_encode(array("send_ticket" => array(
	"recipient" => (string)$to_uin,
	"file_name" => $file_name,
	"file_size" => (string)$file_size,
)));

$headers = array(
	'X-gged-api-version: 6',
	'X-gged-security-token: ' . $security_token
);

$c = curl_init();

curl_setopt($c, CURLOPT_URL, $url);
curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
curl_setopt($c, CURLOPT_HTTPHEADER, $headers);
curl_setopt($c, CURLOPT_COOKIEFILE, COOKIE_FILE); 

curl_setopt($c, CURLOPT_CUSTOMREQUEST, 'PUT'); 
curl_setopt($c, CURLOPT_POSTFIELDS, $data);

$ret = curl_exec($c);

curl_close($c);

if (!$ret)
	die("Failed.\n");
$ret = json_decode($ret);
if (!check_status($ret))
	die("Failed.\n");

$ticket_id = $ret->result->send_ticket->id;
//echo('Ticket ID: ' . $ticket_id . "\n");

/*******************************************************************************
 * Waiting for acknowledgement.
 ******************************************************************************/

function get_status($ticket_id, $security_token)
{
	//echo("Checking status...\n"); flush();
	
	$url = 'https://drive.mpa.gg.pl/send_ticket/' . $ticket_id;
	
	$headers = array(
		'X-gged-api-version: 6',
		'X-gged-security-token: ' . $security_token
	);

	$c = curl_init();

	curl_setopt($c, CURLOPT_URL, $url);
	curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($c, CURLOPT_HTTPHEADER, $headers);
	curl_setopt($c, CURLOPT_COOKIEFILE, COOKIE_FILE); 

	$ret = curl_exec($c);

	curl_close($c);

	if (!$ret)
		die("Failed.\n");
	$ret = json_decode($ret);
	if (!check_status($ret))
		die("Failed.\n");
	
	return $ret->result->send_ticket;
}

while (true)
{
	sleep(1);
	$ts = get_status($ticket_id, $security_token);
	if ($ts->ack_status == 'unknown')
	{
		echo("Waiting for acknowledgement...\n");
		continue;
	}
	if ($ts->ack_status == 'rejected')
		die("Rejected.\n");
	if ($ts->ack_status == 'allowed')
	{
		echo("Accepted.\n");
		break;
	}
	die('Unknown status: ' . $ts->ack_status . "\n");
}

/*******************************************************************************
 * File transfer.
 ******************************************************************************/

echo("Transferring file...\n"); flush();

$url = 'https://drive.mpa.gg.pl/me/file/outbox/' . $ticket_id .
	rawurlencode(',' . $file_name);

$data = json_encode(array(
	'node_type' => 'file',
));

$headers = array(
	'X-gged-metadata: ' . $data,
	'X-gged-local-revision: 0',
	'X-gged-api-version: 6',
	'X-gged-security-token: ' . $security_token
);

$c = curl_init();

curl_setopt($c, CURLOPT_URL, $url);
curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
curl_setopt($c, CURLOPT_HTTPHEADER, $headers);
curl_setopt($c, CURLOPT_COOKIEFILE, COOKIE_FILE); 

curl_setopt($c, CURLOPT_CUSTOMREQUEST, 'PUT'); 
curl_setopt($c, CURLOPT_POSTFIELDS, $file_contents);

$ret = curl_exec($c);

curl_close($c);

if (!$ret)
	die("Failed.\n");
$ret = json_decode($ret);
if (!check_status($ret))
	die("Failed.\n");

echo("File was sent.\n");

unlink(COOKIE_FILE);
